//VERSION 1.3 BY JASON WINNEBECK, FREEWARE, 4/1/99

//Changes from 1.2 to 1.3:
//TOTALLY rewrote this example program! It now emulates a totally unerror-
// checked interpreter.  Directly follows commands as if a program was using
// a LinkList, so if you try to crash it using unsafe functions, you will. If
// you use the safe functions, you'll get the proper safety and error-correct
// provided by the slow functions that one shouldn't need to use if they use
// their loops correctly ;)

//Changes from 1.1 to 1.2:
// Added an #ifdef statement for compatability with non-DJGPP compilers.

#include "linklist.h"
#include <iostream.h>
#include <conio.h>

void DisplayMenu(int cnt, LinkList<int> &mylist, int &resp);
void DisplayAll(LinkList<int> &mylist);  //Must pass by reference because
void DisplayAllO(LinkList<int> &mylist); // LinkLists can't be copied
void DisplayAllL(LinkList<int> &mylist);

int main() {
  clrscr();
  cout << "Read comments in this program to learn about its use." << endl;
  cout << "Read comments in DisplayAllO() to learn about the position variable." << endl;
  cout << "And yes, the crashes ARE built in, and will crash just like it would" << endl;
  cout << " if you had typed in the commands (this acts like an interpreter)." << endl;
  LinkList<int> mylist;
  cout << "Constructed a new LinkList" << endl;
  int resp = 1, temp = 0, cnt = 0;
  while (resp) {
    DisplayMenu(cnt, mylist, resp);
    switch (resp) {
      case 0:
        break;
      case 1:
        cout << "Enter an integer: "; cin >> temp;
        mylist.AddItem(temp);
        break;
      case 2:
        mylist.SetToStart();
        cnt = 0;
        break;
      case 3:
        mylist.Clear();
        cnt = 0;
        break;
      case 4:
        cout << "Enter an integer: "; cin >> temp;
        *mylist._GetNext() = temp;
        break;
      case 5:
        mylist._DeleteThis();
        cnt++;
        break;
      case 6: {
        cout << "Enter an integer: "; cin >> temp;
        int c = 0, len = mylist.Length();
        mylist.SetToStart();
        while ((c < len) && (*mylist._GetNext() > temp)) {
          //Searches until the end or hits a smaller element
          c++;
          mylist._Advance();
        }
        mylist._AddHere(&temp);
        break;}
      case 7:
        mylist._Advance();
        cnt++;
        break;
      case 8:
        DisplayAll(mylist);
        break;
      case 9:
        DisplayAllO(mylist);
        break;
      case 10:
        DisplayAllL(mylist);
        break;
      case 11:
        cout << "Enter element number: "; cin >> temp;
        //Beware!  The following statement WILL cause a GPF depending on the
        // compiler if it's out of bounds.  This is done purposefully to show
        // erronous data comming from NULL
        cout << "Element " << temp << " addr = " << mylist.GetItem(temp) << endl;
#ifndef DJGPP //DJGPP won't balk at NULL.  Remove this line if your compiler
              // won't GPF on a NULL dereference
        if (mylist.GetItem != NULL)
#endif
        cout << "Element " << temp << " data = " << *mylist.GetItem(temp) << endl;
        cout << " GetItem(int) returns NULL if index is out of bounds" << endl;
        break;
      case 12:
        cout << "Enter an integer: "; cin >> temp;
        mylist._AddHere(&temp);
        break;
      case 13:
        int mod;
        cout << "Enter element number: "; cin >> temp;
        cout << "What to modify element " << temp << " to: "; cin >> mod;
        mylist[temp] = mod;
        cout << " Changed element.  If index was out of range, changed last element in list" << endl;
        break;
      default:
        cout << "Wrong number!" << endl;
        getch();
        break;
    }
  }

  return 0;
}

void DisplayMenu(int cnt, LinkList<int> &mylist, int &resp) {
  cout << "Done.  Press a key" << endl;
  getch();
  clrscr();
  cout << "LinkList Length: " << mylist.Length() << "  Position: " << cnt << "  IsAtEnd? " << mylist.IsAtEnd() << endl;
  cout << " Position emulates a for loop and is not nessacarily the true element number" << endl;
  if (!mylist.IsAtEnd())
    cout << "Current element addr: " << mylist._GetNext() << " data:" << *mylist._GetNext() << endl;
  else
    cout << "The list has gone over the end and there is no element to display." << endl;

  cout << "0  - Quit" << endl;
  cout << "1  - Add an Integer using AddItem()" << endl;
  cout << "2  - Go back to start of the list using SetToStart()" << endl;
  cout << "3  - Clear list and start anew using Clear()" << endl;
  cout << "4  - Modify this element using *_GetNext() assignment" << endl;
  cout << "5  - Delete this item using _DeleteThis()" << endl;
  cout << "6  - Add a new sorted element using a search loop and _AddHere()" << endl;
  cout << "7  - _Advance() to the next item in the list" << endl;
  cout << "8  - Display all elements unoptimized using bool GetNext()" << endl;
  cout << "9  - Display all elements optimized using _GetNext() and _Advance()" << endl;
  cout << "10 - Display all elements by making a linear array and for loop" << endl;
  cout << "11 - Display element x using GetItem(int)" << endl;
  cout << "12 - Add an element before this one with _AddHere()" << endl;
  cout << "13 - Modify element x using [] operator" << endl;
  cout << endl;
  cout << "Enter decision: "; cin >> resp;
  cout << endl;
}

void DisplayAll(LinkList<int> &mylist) {
  mylist.SetToStart();         //Set to beginning of the list
  int temp, numb = 0;          //temp vars
  while (mylist.GetNext(temp)){//Until the end of the list
    cout << "Element " << numb << ": " << temp << endl;
    numb++;
  }
}

void DisplayAllO(LinkList<int> &mylist) {
  mylist.SetToStart();
  int len = mylist.Length(); //Don't want to recompute the length each for...
                             // plus, deleting an element will decrement the
                             // length, and make us lose track in the list --
                             // this also goes for adding, etc.
  for(int c=0; c < len; c++) {
    //_GetNext() returns un-copied pointer.  You can read data, change it,
    // whatever you want to do with it, then move on!!!
    cout << "Element " << c << ": " << *mylist._GetNext() << endl;
    mylist._Advance();
  }
  //When traversing the LinkList, using optimized funcs only (it's unsafe to
  // mix the two groups) additions will always be inserted BEFORE
  // the current element, and therefore don't need to be considered in the
  // for count.  However, deletion will, but if you treat _DeleteThis the same
  // as the _Advance, all should be fine like this:

  // for (int c=0; c<len; c++)
  //   if(DoWeWantToDelete(mylist._GetNext()))
  //     mylist._DeleteThis();
  //   else
  //     mylist._Advance();

  //The counter variable emulates if a for loop was used in this fashion
}

void DisplayAllL(LinkList<int> &mylist) {
  int *myarr = mylist.MakeLinear(); //Get a linear array
  int len = mylist.Length();        //Length of linear array
  for (int c=0; c<len; c++)
    cout << "Element " << c << ": " << myarr[c] << endl;
  delete myarr;                   //We're done with the new data
}
